/*
 * Uloha1Test.java
 * JUnit based test
 *
 */
package cz.muni.fi.pb138.jaro2011.uloha1;

import java.io.IOException;
import java.net.URI;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.dom.Document;

/**
 *
 * @author originally Petr Adamek 2008, changes by Tomas Pitner, Feb 2011
 */
public class Uloha1Test extends TestCase {

    public Uloha1Test(String testName) {
        super(testName);
    }
    Main reseniUlohy1;
    XPath xpath;

    protected void setUp() throws Exception {
        URI uri = Uloha1Test.class.getResource("people.xml").toURI();
        if (uri == null) {
            throw new IOException("Nelze nalezt testovaci XML soubor");
        }
        reseniUlohy1 = Main.newInstance(uri);

        xpath = XPathFactory.newInstance().newXPath();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(Uloha1Test.class);
        return suite;
    }

    /**
     * Test of changeName method, of class Main
     */
    public void testReduceSalaryByFine() throws XPathExpressionException {
        reseniUlohy1.reduceSalaryByFine("6B6 82-51", 20000.0);
        assertEquals(68475.25 - 20000.0, getSalary("3"));

        reseniUlohy1.reduceSalaryByFine("ZNC 33-93", 2000.0);
        assertEquals(12000.0 - 2000.0, getSalary("1"));
    }

    /**
     * Test of getWageCosts method, of class Main
     */
    public void testGetAverageCarAge() throws XPathExpressionException {
        assertEquals(getAverageCarAge(reseniUlohy1.getDocument()), reseniUlohy1.getAverageCarAge());
    }

    private double getAverageCarAge(Document doc) throws XPathExpressionException {
        return ((Double) xpath.evaluate(
                "sum(//year)", reseniUlohy1.getDocument(), XPathConstants.NUMBER)) / ((Double) xpath.evaluate(
                "count(//year)", reseniUlohy1.getDocument(), XPathConstants.NUMBER));
    }

    public void testRemoveCarsProducedIn1995() throws XPathExpressionException {
        assertEquals("There should be just two cars produced in 1995 removed.",
                2, reseniUlohy1.removeCarsProducedIn(1995));
        assertEquals("3 cars should be present after removal.",
                3, getCarsCount());
    }

    public void testRemoveCarsProducedIn2009() throws XPathExpressionException {
        assertEquals(1, reseniUlohy1.removeCarsProducedIn(2009));
        assertEquals("4 cars should be present after removal.",
                4, getCarsCount());
    }

    private int getOlderCarsCount(int year) throws XPathExpressionException {
        double carsWithoutYear = (Double) xpath.evaluate("count(//car[not(year)])",
                reseniUlohy1.getDocument(), XPathConstants.NUMBER);
        if (carsWithoutYear > 0) {
            fail("You have just removed year child element, not the whole car!");
        }
        double older = (Double) xpath.evaluate("count(//car[year<" + year + " and count(year)=1])",
                reseniUlohy1.getDocument(), XPathConstants.NUMBER);
        return (int) older;
    }

    private int getCarsCount() throws XPathExpressionException {
        double carsCount = (Double) xpath.evaluate("count(//car)",
                reseniUlohy1.getDocument(), XPathConstants.NUMBER);
        return (int) carsCount;
    }

    private double getSalary(String id) throws XPathExpressionException {
        return (Double) xpath.evaluate("number(//person[@id='" + id + "']/salary/text())",
                reseniUlohy1.getDocument(), XPathConstants.NUMBER);
    }
}
